----------------------------------------
-- CPU Top Level
----------------------------------------

-------------------------------------------------------------------------------
-- Notes
--  2007.01.30
---- CX^XLqB
-- 2007.01.31
---- ʂLqIBSELMregfile/sramƂEXBUSYI𒆁B
-- 2007.02.11
---- SRAMcomponentLqȂ̂͂ȂIt܂B
---- USBcomponentɑ܂B
---- ̉HōsĂ邱Ƃ́ACX^X̐ڑƁACLKMO,CLKM1(SRAM),RSTX(USB)̋LqB
-------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity CPUTOP is
    port(CLKIN : in std_logic;   -- clock 50MHz
        LED : out std_logic_vector(7 downto 0);
        -- to SRAM
        SRAM_RW : out std_logic; -- read or write
        SRAM_IO : inout std_logic_vector(31 downto 0);  -- IO data
        SRAM_IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        SRAM_AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
        SRAM_CLKM0,SRAM_CLKM1 : out std_logic;                      -- SRAM clock
        SRAM_BW : out std_logic_vector(3 downto 0);
        SRAM_CEH1X,SRAM_CEL1X,SRAM_CE2,SRAM_CE2X : out std_logic;
        SRAM_OEX,SRAM_CENX,SRAM_ADVLD : out std_logic;
        SRAM_LBOX : out std_logic;
        SRAM_ZZ : out std_logic;
        -- usb
        USB_RSTX : out std_logic;                        -- USB reset
        USB_RDX : out std_logic;                         -- 0 when read
        USB_WR : out std_logic;                          -- 1 when write
        USB_TXEX : in std_logic;                         -- 1 when buffer is full
        USB_RXFX : in std_logic;                         -- 1 when buffer is empty
        USB_DATA : inout std_logic_vector(7 downto 0)   -- USB data
        );

end CPUTOP;

architecture ARC of CPUTOP is

    component ROC
        port(O : out std_logic);
    end component;

    component IBUFG
        port(I : in std_logic;
            O : out std_logic);
    end component;

    component SRAM
        port(
        CLK : in std_logic;                         -- clock 50MHz
        RESET : in std_logic;                       -- reset

        MODE : in std_logic;                       -- read/write mode
        ADDR : in std_logic_vector(19 downto 0);    -- read/write address
        AOUT : out std_logic_vector(19 downto 0);   -- current data address
        RDATA : out std_logic_vector(31 downto 0);  -- read data
        RP : out std_logic_vector(3 downto 0);      -- read parity
        WDATA : in std_logic_vector(31 downto 0);   -- write data
        WP : in std_logic_vector(3 downto 0);       -- write parity
        -- to SRAM
        RW : out std_logic; -- read or write
        IO : inout std_logic_vector(31 downto 0);  -- IO data
        IOP : inout std_logic_vector(3 downto 0);  -- IO parity
        AA : out std_logic_vector(19 downto 0);    -- address
        -- to SRAM(constant)
--        CLKM0,CLKM1 : out std_logic;                      -- SRAM clock
        BW : out std_logic_vector(3 downto 0);
        CEH1X,CEL1X,CE2,CE2X : out std_logic;      -- chip select
        OEX,CENX,ADVLD : out std_logic;
        LBOX : out std_logic;
        ZZ : out std_logic);
    end component;

    component IFDR
        port(CLK : in std_logic;     -- system clock
        RESET : in std_logic;    -- reset
        -- Mr.SRAM
        PC : out std_logic_vector(19 downto 0);  -- instruction address(PC) ->sram & EXW
        -- SRAM
        INST : in std_logic_vector(31 downto 0); -- instruction
        -- EXW
        EXPC : in std_logic_vector(19 downto 0);   -- External PC
        BUSY : in std_logic;                       -- EXW busy signal
        VALID : out std_logic;                    -- FDR valid
        OP1 : out std_logic_vector(1 downto 0);   -- opcode1
        OP2 : out std_logic_vector(3 downto 0);   -- opcode2
        REGAD : out std_logic_vector(6 downto 0);  -- desg reg addr/imm7,disp7
        IMMSEL : out std_logic;                    -- if IMM then 1 else 0
        IMM : out std_logic_vector(31 downto 0);   -- imm
        -- register file
        REG_ADS : out std_logic_vector(6 downto 0); -- R2 addr
        REG_ADT : out std_logic_vector(6 downto 0) -- R3 addr
        );
    end component;

    component EXW
        port(
        CLK : in std_logic;     -- system clock
        RESET : in std_logic;    -- reset
        -- Mr.SRAM
        SRAM_RW : out std_logic;                        --sram rw
        SRAM_ADDR : out std_logic_vector(19 downto 0);  -- sram addr
        -- SRAM
        SRAM_WD : out std_logic_vector(31 downto 0); -- to sram, write data
        SRAM_RD : in std_logic_vector(31 downto 0); -- from Sram, read data
        -- IFDR
        PC : in std_logic_vector(19 downto 0);  -- instruction address(PC)
        EXPC : out std_logic_vector(19 downto 0);   -- External PC
        BUSY : out std_logic;                       -- EXW busy signal
        VALID : in std_logic;                    -- FDR valid
        OP1 : in std_logic_vector(1 downto 0);   -- opcode1
        OP2 : in std_logic_vector(3 downto 0);   -- opcode2
        REGAD : in std_logic_vector(6 downto 0);  -- desg reg addr/imm7,disp7
        IMMSEL : in std_logic;                    -- if IMM then 1 else 0
        IMM : in std_logic_vector(31 downto 0);   -- imm
        -- register file
--        REGSEL : out std_logic;                  -- select signal 1->EXW 0->FDR
        REGWE : out std_logic;                    -- regfile write enable(0->read)
        REGAD_D : out std_logic_vector(6 downto 0); -- dest reg addr
        REG_D : out std_logic_vector(31 downto 0); -- regfile write data
        REG_S : in std_logic_vector(31 downto 0); -- R2 src data
        REG_T : in std_logic_vector(31 downto 0); -- R3/R1 src data
        -- usb
        USB_RMODE : out std_logic;
        USB_WMODE : out std_logic;
        USB_RDATA : in std_logic_vector(7 downto 0);
        USB_WDATA : out std_logic_vector(7 downto 0);
        USB_VALID : in std_logic;
        USB_BUSY  : in std_logic
        );
    end component;

    component MRSRAM
        port(SEL : in std_logic;   -- select signal IFDR->0,EXW->1
          IFDR_ADDR : in std_logic_vector(19 downto 0);
          EXW_MODE : in std_logic;
          EXW_ADDR : in std_logic_vector(19 downto 0);
          -- to&from SRAM controller
          MODE : out std_logic;
          ADDR : out std_logic_vector(19 downto 0)
        );
    end component;

    component REGFILE
        port(
            CLK   : in  std_logic;                       -- system clock
            SEL   : in std_logic;                        -- select '1'->EXW '0'->IFDR
            -- EXW
            REG_OA : out std_logic_vector(31 downto 0);   -- src reg1
            REG_OB : out std_logic_vector(31 downto 0);   -- src reg2
            REG_IB : in  std_logic_vector(31 downto 0);   -- dest reg
            WEX     : in std_logic;                 -- write enable
            REGAD_D : in std_logic_vector(6 downto 0);    -- dest/src reg addr
            -- IFDR
            REGAD_S : in std_logic_vector(6 downto 0);  -- src reg1 addr
            REGAD_T : in std_logic_vector(6 downto 0)  -- src reg2 addr
        );
    end component;

    component USB
        port(
        CLK : in std_logic;                          -- clock 50MHz
        RESET : in std_logic;                        -- reset signal
        RMODE : in std_logic;                        -- 1 when read
        WMODE : in std_logic;                        -- 1 when write
        RDATA : out std_logic_vector(7 downto 0);    -- read data
        WDATA : in std_logic_vector(7 downto 0);     -- write data
        VALID : out std_logic;                       -- 1 when read is O.K.
        BUSY : out std_logic;
        -- to USB
        --RSTX : out std_logic;                        -- USB reset
        RDX : out std_logic;                         -- 0 when read
        WR : out std_logic;                          -- 1 when write
        TXEX : in std_logic;                         -- 1 when buffer is full
        RXFX : in std_logic;                         -- 1 when buffer is empty
        USBDATA : inout std_logic_vector(7 downto 0));   -- USB data
    end component;

    signal CLK : std_logic;     -- inner clock;
    signal RESET : std_logic;   -- reset

    -- inst sram
    signal SAOUT : std_logic_vector(19 downto 0);
    signal SMODE : std_logic;  -- mrsram -> sram
    signal SADDR : std_logic_vector(19 downto 0);  -- mrsram -> sram
    signal SRDATA : std_logic_vector(31 downto 0);  -- sram -> ifdr,exr
    signal SRP : std_logic_vector(3 downto 0);
    signal SWDATA : std_logic_vector(31 downto 0); -- exr -> sram
    signal SWP : std_logic_vector(3 downto 0);
    signal SWR : std_logic;                        -- mrsram -> sram

    -- inst ifdr & exw & mrsram & regfile
    signal SIN_IFDR : std_logic_vector(19 downto 0);
    signal EXPC : std_logic_vector(19 downto 0);
    signal EXBUSY : std_logic;
    signal FDRVAL : std_logic;
    signal OP1 : std_logic_vector(1 downto 0);
    signal OP2 : std_logic_vector(3 downto 0);
    signal REGAD : std_logic_vector(6 downto 0);
    signal IMMSEL : std_logic;
    signal IMM : std_logic_vector(31 downto 0);
    signal REGAD_D,REGAD_S,REGAD_T : std_logic_vector(6 downto 0);
    signal REGWE : std_logic;
    signal REG_D,REG_S,REG_T : std_logic_vector(31 downto 0);
    -- inst exw & mrsram
    signal SINRW_EXW : std_logic;
    signal SIN_EXW : std_logic_vector(19 downto 0);

    -- usb
    signal USB_WMODE : std_logic;
    signal USB_RMODE : std_logic;
    signal USB_WDATA : std_logic_vector(7 downto 0);
    signal USB_RDATA : std_logic_vector(7 downto 0);
    signal USB_BUSY : std_logic;
    signal USB_VALID : std_logic;

begin
    RC : ROC port map (
        O => RESET);

    -- Inner clk is "CLK"
    IBG : IBUFG port map (
        I => CLKIN,
        O => CLK);

    SRAM_INST : SRAM port map (
        CLK   => CLK,
        RESET => RESET,
        MODE  => SMODE,
        ADDR  => SADDR,
        AOUT  => SAOUT,
        RDATA => SRDATA,
        RP    => SRP,
        WDATA => SWDATA,
        WP    => SWP,
        RW    => SRAM_RW,
        IO    => SRAM_IO,
        IOP   => SRAM_IOP,
        AA    => SRAM_AA,
--        CLKM0  => SRAM_CLKM0,
--        CLKM1 => SRAM_CLKM1,
        BW    => SRAM_BW,
        CEH1X  => SRAM_CEH1X,
        CEL1X  => SRAM_CEL1X,
        CE2   => SRAM_CE2,
        CE2X  => SRAM_CE2X,
        OEX   => SRAM_OEX,
        CENX  => SRAM_CENX,
        ADVLD => SRAM_ADVLD,
        LBOX  => SRAM_LBOX,
        ZZ    => SRAM_ZZ);

    IFDR_INST : IFDR
        port map(
        CLK => CLK,
        RESET => RESET,
        -- Mr.SRAM
        PC => SIN_IFDR,
        -- SRAM
        INST => SRDATA,
        -- EXW
        EXPC => EXPC,
        BUSY => EXBUSY,
        VALID => FDRVAL,
        OP1 => OP1,
        OP2 => OP2,
        REGAD =>REGAD,
        IMMSEL =>IMMSEL,
        IMM =>IMM,
        -- register file
        REG_ADS => REGAD_S,
        REG_ADT => REGAD_T

        );

    EXW_INST : EXW port map(
        CLK => CLK,
        RESET => RESET,
        -- Mr.sram
        SRAM_RW => SINRW_EXW,
        SRAM_ADDR => SIN_EXW,
        -- sram
        SRAM_WD => SWDATA,
        SRAM_RD => SRDATA,
        -- ifdr
        PC => SIN_IFDR,
        EXPC => EXPC,
        BUSY => EXBUSY,
        VALID => FDRVAL,
        OP1 => OP1,
        OP2 => OP2,
        REGAD => REGAD,
        IMMSEL => IMMSEL,
        IMM => IMM,
        -- regfile
        REGWE => REGWE,
        REGAD_D => REGAD_D,
        REG_D => REG_D,
        REG_S => REG_S,
        REG_T => REG_T,
        -- usb
        USB_RMODE => USB_RMODE,
        USB_WMODE => USB_WMODE,
        USB_RDATA => USB_RDATA,
        USB_WDATA => USB_WDATA,
        USB_BUSY => USB_BUSY,
        USB_VALID => USB_VALID
        );

    MRSRAM_INST : MRSRAM port map(
        SEL => EXBUSY,
        IFDR_ADDR => SIN_IFDR,
        EXW_MODE => SINRW_EXW,
        EXW_ADDR => SIN_EXW,
        MODE => SMODE,
        ADDR => SADDR
        );

    REGFILE_INST : REGFILE port map(
        CLK => CLK,
        SEL => EXBUSY,
        REG_OA => REG_S,
        REG_OB => REG_T,
        REG_IB => REG_D,
        WEX => REGWE,
        REGAD_D =>REGAD_D,
        REGAD_S =>REGAD_S,
        REGAD_T =>REGAD_T
    );

    USB_INST : USB port map(
        CLK => CLK,
        RESET => RESET,
        RMODE => USB_RMODE,
        WMODE => USB_WMODE,
        RDATA => USB_RDATA,
        WDATA => USB_WDATA,
        VALID => USB_VALID,
        BUSY  => USB_BUSY,
        -- to USB
        --RSTX : out std_logic;                        -- USB reset
        RDX => USB_RDX,
        WR => USB_WR,
        TXEX => USB_TXEX,
        RXFX => USB_RXFX,
        USBDATA => USB_DATA
    );

USB_RSTX <= '0' when RESET = '1' else
            '1';
SRAM_CLKM0 <= CLK;
SRAM_CLKM1 <= CLK;
SWP <= "0000";

end ARC;
