-- FINVSQRT
-- 平方根の逆数

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT is

	port(SRC    : in std_logic_vector(31 downto 0);
	     OUTPUT : out std_logic_vector(31 downto 0));

end FINVSQRT;

architecture STRUCTURE of FINVSQRT is

component FDIV
   port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
            OUTPUT : out std_logic_vector(31 downto 0));
end component;
component SQRT
   port(SRC    : in std_logic_vector(31 downto 0);
	     OUTPUT : out std_logic_vector(31 downto 0));
end component;

signal NaNs : std_logic;      -- Not a Number
signal Inf_p : std_logic;     -- '1' when SRC is positive Infinity
signal SQRT_SRC : std_logic_vector(31 downto 0);
signal FIVSQ : std_logic_vector(31 downto 0);

begin -- STRUCTURE
   NaNs <= '1' when SRC(31) = '1' or SRC(30 downto 0) = "0000000000000000000000000000000" 
                 or (SRC(30 downto 23) = 255 and SRC(22 downto 0) /= 0) else -- SRC =< 0
           '0';
   Inf_p <= '1' when SRC = "01111111100000000000000000000000" else
            '0';
   
   SQRT_FINVSQRT : SQRT port map (SRC,SQRT_SRC);
   FDIV_FINVSQRT : FDIV port map (SQRT_SRC,SRC,FIVSQ);
   
   OUTPUT <= "01111111110000000000000000000000" when NaNs = '1' else -- NaNs
             "00000000000000000000000000000000" when Inf_p = '1' else
             FIVSQ;
                        
end STRUCTURE;
