-- FINVSQRT
-- ニュートン法による平方根の逆数

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FINVSQRT_n is

	port(SRC    : in std_logic_vector(31 downto 0);
	     OUTPUT : out std_logic_vector(31 downto 0));

end FINVSQRT_n;

architecture STRUCTURE of FINVSQRT_n is

component FINVSQRT_table is
--	port(M    : in std_logic_vector(8 downto 0);
--	     MANT : out std_logic_vector(7 downto 0));
	port(M    : in std_logic_vector(7 downto 0);
	     MANT : out std_logic_vector(6 downto 0));
end component;

component FINVSQRT_repeat is
	port(X      : in std_logic_vector(24 downto 0);    --MSB含む
	     A      : in std_logic_vector(24 downto 0);
	     nextX : out std_logic_vector(24 downto 0));
end component;

signal NaNs : std_logic;      -- Not a Number
signal Inf_p : std_logic;     -- '1' when SRC is positive Infinity
--signal SQRT_SRC : std_logic_vector(31 downto 0);
--signal FIVSQ : std_logic_vector(31 downto 0);

signal A : std_logic_vector(24 downto 0);
signal S : std_logic;
--signal EXP_buf : std_logic_vector(7 downto 0);
signal S_EXP : std_logic_vector(7 downto 0);
signal EXP : std_logic_vector(7 downto 0);
signal SMANT0 : std_logic; --'0' when mantissa of SRC is 0
--signal M_buf : std_logic_vector(7 downto 0);
signal M_buf : std_logic_vector(6 downto 0);
signal M_buf1 : std_logic_vector(24 downto 0);
signal M_buf2 : std_logic_vector(24 downto 0);
signal M_buf3 : std_logic_vector(24 downto 0);
--signal M_buf4 : std_logic_vector(24 downto 0);
signal MANT : std_logic_vector(22 downto 0);
--signal O : std_logic;
--signal Os : std_logic_vector(8 downto 0);

begin -- STRUCTURE
   NaNs <= '1' when SRC(31) = '1' or SRC(30 downto 0) = "0000000000000000000000000000000" 
                 or (SRC(30 downto 23) = 255 and SRC(22 downto 0) /= 0) else -- SRC =< 0
           '0';
   Inf_p <= '1' when SRC = "01111111100000000000000000000000" else
            '0';
   
   S <= SRC(31);
   S_EXP <= SRC(30 downto 23) - 127;
   EXP <= (not (S_EXP(7) & S_EXP(7 downto 1))) + (not S_EXP(0)) + 127 when SRC(22 downto 0) = 0 else
          (not (S_EXP(7) & S_EXP(7 downto 1))) + 127;
   --EXP <= EXP_buf when SRC(22 downto 0) = 0 else EXP_buf - 1;
   
   --SMANT0 <= '0' when SRC(22 downto 0) = 0 else '1';
   A <= '1' & SRC(22 downto 0) & '0';--SMANT0 & SRC(22 downto 0) & '0';
   --FINVSQRT_table7_1 : FINVSQRT_table port map (SRC(23 downto 15),M_buf);
   FINVSQRT_table7_1 : FINVSQRT_table port map (SRC(23 downto 16),M_buf);
   --M_buf1 <= '1' & M_buf & "0000000000000000";
   M_buf1 <= '1' & M_buf & "00000000000000000";
   FINVSQRT_repeat1 : FINVSQRT_repeat port map (M_buf1,A,M_buf2);
   FINVSQRT_repeat2 : FINVSQRT_repeat port map (M_buf2,A,M_buf3);
   
   MANT <= (others => '0') when (SRC(22 downto 0) = 0 and SRC(23) = '1') else
           M_buf3(23 downto 1) + 1 when M_buf3(0) = '1' else  --丸め
           M_buf3(23 downto 1);
   
   --O <= NaNs or (not Inf_p);
   --Os <= (others => O);
   OUTPUT <= "01111111110000000000000000000000" when NaNs = '1' else -- NaNs
             "00000000000000000000000000000000" when Inf_p = '1' else
             --'0' & Os & "0000000000000000000000" when (NaNs or Inf_p) = '1' else
             S & EXP & MANT;
                        
end STRUCTURE;
