-- FSUB_S
-- ソースオペランドの符合は一致しているものとする

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FSUB_S is

	port(SRC1,SRC2 : in std_logic_vector(31 downto 0);
	     OUTPUT    : out std_logic_vector(31 downto 0);
	     NaNout    : out std_logic;
	     ZeroOut   : out std_logic);

end FSUB_S;

architecture STRUCTURE of FSUB_S is
component RightShifter24_2
	port(Data    : in std_logic_vector(23 downto 0);
	     nShift  : in std_logic_vector(7 downto 0);
	     Shifted : out std_logic_vector(26 downto 0));
end component;
component Priority31_sub
       port(bits : in std_logic_vector(31 downto 0);
            Code_out : out std_logic_vector(4 downto 0));
end component;
component LeftShifter31_sub
    	  port(Data    : in std_logic_vector(31 downto 0);
    	       nShift   : in std_logic_vector(4 downto 0);
	         Shifted : out std_logic_vector(22 downto 0));
end component;

signal NaN : std_logic;                       -- Not a Number
--signal Inf : std_logic;                       -- Infinity
signal Inf1 : std_logic;                       --SRC1 is Infinity
signal Inf2 : std_logic;                       --SRC2 is Infinity

signal S : std_logic;
signal W : std_logic_vector(31 downto 0);     -- W数
signal L : std_logic_vector(31 downto 0);     -- L数
signal Data : std_logic_vector(23 downto 0);  -- input to RightShifter
signal nShift : std_logic_vector(7 downto 0); -- input to RightShifter
signal EXP : std_logic_vector(7 downto 0);
signal L_MANT_buf : std_logic_vector(26 downto 0);
signal MANT_buf : std_logic_vector(26 downto 0);
signal MANT_buf2 : std_logic_vector(26 downto 0);    -- MANT_bufを丸め
signal MANT_INPUT : std_logic_vector(31 downto 0); -- input to Priority31,LeftShifter31
signal POSITION_MSB : std_logic_vector(4 downto 0);  -- MANT_buf2の先頭の1の位置
signal MANT : std_logic_vector(22 downto 0);

signal OUTEXPis0 : std_logic;
signal O : std_logic;
signal Os : std_logic_vector(8 downto 0);

signal InfS : std_logic;
begin -- STRUCTURE
   NaN <= '1' when (SRC1(30 downto 23) = 255 and SRC1(22 downto 0) /= 0) or 
                   (SRC2(30 downto 23) = 255 and SRC2(22 downto 0) /= 0) else
          '0';
   Inf1 <= '1' when SRC1(30 downto 0) = "1111111100000000000000000000000" else
           '0';
   Inf2 <= '1' when SRC2(30 downto 0) = "1111111100000000000000000000000" else
           '0';
   
   S <= '1' when ((SRC1(31) = '0' and (SRC1(30 downto 0) < SRC2(30 downto 0)))
               or (SRC1(31) = '1' and (SRC1(30 downto 0) > SRC2(30 downto 0)))) else
        '0';        
   W <= SRC2 when (SRC1(30 downto 0) < SRC2(30 downto 0)) else
        SRC1;
   L <= SRC1 when (SRC1(30 downto 0) < SRC2(30 downto 0)) else
        SRC2;
   Data <= '1' & L(22 downto 0);
   
   nShift <= W(30 downto 23) - L(30 downto 23);
   
   --RightShifter24_2_1 : RightShifter24_2 port map (Data,nShift,L_MANT_buf); -- Ｌ数の仮数部を丸めつつ右シフト
   L_MANT_buf <= shr(Data & "000",nShift);
   
   
   MANT_buf <= (not L_MANT_buf) + ('1' & W(22 downto 0) & "000");           -- 仮の仮数部となる
   MANT_buf2 <= (MANT_buf(26 downto 3) + "000000000000000000000001") & "000"
                   when (MANT_buf(26) = '1' and          -- 仮数部の引き算で先頭の'1'が動いていない場合の丸め
                        MANT_buf(2) = '1') else
                (MANT_buf(26 downto 2) + "0000000000000000000000001") & "00"
                   when (MANT_buf(26) = '0' and          -- 仮数部の引き算で先頭の'1'が動いている場合の丸め
                        MANT_buf(1) = '1') else
                MANT_buf;                                -- 次のプライオリティエンコーダで揃えるので、丸めない場合は揃え不要
   MANT_INPUT <= "00000" & MANT_buf2;
   Priority31_2 : Priority31_sub port map (MANT_INPUT,POSITION_MSB);
   EXP <= W(30 downto 23) - ("11010" - POSITION_MSB);    -- (Ｗ数の指数) - (26-(プライオリティエンコーダの出力))
   
   
   LeftShifter31_2 : LeftShifter31_sub port map (MANT_INPUT,POSITION_MSB,MANT);
   --MANT <= shl(MANT_INPUT,(not POSITION_MSB));
   
   OUTEXPis0 <= '1' when EXP = 0 else '0';
   
   NaNout <= NaN or (Inf1 and Inf2);-- or OUTEXPis0;
   ZeroOut <= OUTEXPis0;
   
   O <= NaN or (Inf1 and Inf2) or (not OUTEXPis0);
   Os <= (others => O);
   InfS <= SRC1(31) when Inf1 = '1' else (not SRC2(31));
   OUTPUT <= ----"01111111110000000000000000000000"                 when NaN = '1' or (Inf1 = '1' and Inf2 = '1') else
             --Os & "0000000000000000000000"     when (NaN or (Inf1 and Inf2) or OUTEXPis0) = '1' else
             --"1111111100000000000000000000000" when Inf1 = '1' else
             --"1111111100000000000000000000000" when Inf2 = '1' else
             ----S & "1111111100000000000000000000000"              when EXP = 255 else -- ? 同じ符号同士の引き算なので正規数での指数最大値は254
             ----"00000000000000000000000000000000"                 when OUTEXPis0 = '1' else -- ?
             --EXP & MANT                                 -- MANT(31 downto 9) = MANTISSA
             --  when W(30 downto 0) /= L(30 downto 0) else
             --"00000000" & MANT;
             
             
             --"01111111110000000000000000000000"                 when NaN = '1' or (Inf1 = '1' and Inf2 = '1') else
             '0' & Os & "0000000000000000000000"                when (NaN or (Inf1 and Inf2) or OUTEXPis0) = '1' else
             --SRC1(31) & "1111111100000000000000000000000"       when Inf1 = '1' else
             --(not SRC2(31)) & "1111111100000000000000000000000" when Inf2 = '1' else
             InfS & "1111111100000000000000000000000"           when (Inf1 or Inf2) = '1' else
             --S & "1111111100000000000000000000000"              when EXP = 255 else -- ? 同じ符号同士の引き算なので正規数での指数最大値は254
             --"00000000000000000000000000000000"                 when OUTEXPis0 = '1' else -- ?
              S & EXP & MANT                                 -- MANT(31 downto 9) = MANTISSA
               when W(30 downto 0) /= L(30 downto 0) else
             S & "00000000" & MANT;
end STRUCTURE;
