-- FTOI
-- 小数点以下四捨五入

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity FTOI is

	port(SRC    : in std_logic_vector(31 downto 0);
	     OUTPUT : out std_logic_vector(31 downto 0));

end FTOI;

architecture STRUCTURE of FTOI is
component LeftShifter24to54
    	  port(Data    : in std_logic_vector(23 downto 0);
	         nShift  : in std_logic_vector(4 downto 0);
	         Shifted : out std_logic_vector(53 downto 0));
end component;

signal NaN : std_logic;                       -- Not a Number

signal S : std_logic;
signal EXP : std_logic_vector(7 downto 0);
signal SRC_MANT : std_logic_vector(23 downto 0);        -- input to LeftShifter24to54
signal nShift : std_logic_vector(4 downto 0);           -- input to LeftShifter24to54
signal INT_abs : std_logic_vector(31 downto 0);         -- 末尾INT_abs(0)は小数点以下
signal INT_abs_buf : std_logic_vector(53 downto 0);
signal INT_abs_rounded : std_logic_vector(30 downto 0); -- INT_absを四捨五入
signal EXP7s : std_logic_vector(29 downto 0);
signal Zero : std_logic;
 
begin -- STRUCTURE
   NaN <= '1' when (SRC(30 downto 23) = 255 and SRC(22 downto 0) /= 0) else
          '0';
   
   S <= SRC(31) when SRC /= "10000000000000000000000000000000" else
        '0';
   EXP <= SRC(30 downto 23);
   Zero <= '1' when EXP <= 125 else '0';
   SRC_MANT <= '1' & SRC(22 downto 0);
   nShift <= EXP(4 downto 0) + 1;
   
   --LeftShifter24to54_1 : LeftShifter24to54 port map (SRC_MANT,nShift,INT_abs_buf);
   INT_abs_buf <= shl("000000000000000000000000000000" & SRC_MANT,nShift);
   
   EXP7s <= (others => EXP(7));
   INT_abs <= "00000000000000000000000000000000" 
                  --when EXP >= 158 and S = '1' else         -- INT_MIN
                  when Zero = '1' or (EXP >= 158 and S = '1') else         -- less than 0.5 or INT_MIN
              --   when EXP <= 125 else
              EXP7s & "10" when EXP = 126 or (EXP >= 158 and S = '0') else -- 0.5~1.0 or INT_MAX
              --"00000000000000000000000000000010" when EXP  = 126 else -- 四捨五入した結果
              --"11111111111111111111111111111110" when (EXP >= 158 and S = '0') else -- INT_MAX
              --"00000000000000000000000000000000" when (EXP >= 158 and S = '1') else -- INT_MIN
              INT_abs_buf(53 downto 22); -- 127 <= EXP <= 157
   INT_abs_rounded <= INT_abs(31 downto 1) when INT_abs(0) = '0' else
                      INT_abs(31 downto 1) + 1;
   OUTPUT <= "00000000000000000000000000000000"  when NaN = '1' else
             '0' & INT_abs_rounded               when S = '0' else
             ----(('1' & (not INT_abs_rounded)) + 1) when EXP(7) = '0' else  -- -0.0 ~ -0.5(EXP<=125)
             (Zero xor '1') & ((not INT_abs_rounded) + 1);     -- INT_MIN
             
             --"00000000000000000000000000000000"  when (NaN or Zero) = '1' else
             --'0' & INT_abs_rounded               when S = '0' else
             --'1' & ((not INT_abs_rounded) + 1);     -- INT_MIN

             ----(('1' & (not INT_abs_rounded)) + 1) when Zero = '1' else -- -0.0 ~ -0.5(EXP<=125)
      
end STRUCTURE;
