-- INV
-- 初期値を7bitとり、2度計算する
-- 

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity INV7 is

	port(SRC    : in std_logic_vector(31 downto 0);
	     OUTPUT : out std_logic_vector(31 downto 0));

end INV7;

architecture STRUCTURE of INV7 is

component INV_table7 is
	port(M    : in std_logic_vector(7 downto 0);
	     MANT : out std_logic_vector(6 downto 0));
end component;

component INV_repeat is
	port(X      : in std_logic_vector(24 downto 0);    --MSB含む
	     A      : in std_logic_vector(24 downto 0);
	     nextX : out std_logic_vector(24 downto 0));
end component;

signal NaN  : std_logic;                  -- OUTPUT is Not a Number
signal Inf  : std_logic;                  -- OUTPUT is Infinity
signal Zero : std_logic;                  -- OUTPUT is 0

signal A : std_logic_vector(24 downto 0);
signal S : std_logic;
signal EXP_buf : std_logic_vector(7 downto 0);
signal EXP : std_logic_vector(7 downto 0);
signal SMANT0 : std_logic; --'0' when mantissa of SRC is 0
signal M_buf : std_logic_vector(6 downto 0);
signal M_buf1 : std_logic_vector(24 downto 0);
signal M_buf2 : std_logic_vector(24 downto 0);
signal M_buf3 : std_logic_vector(24 downto 0);
--signal M_buf4 : std_logic_vector(24 downto 0);
signal MANT : std_logic_vector(22 downto 0);
signal O : std_logic;
signal Os : std_logic_vector(8 downto 0);
begin -- STRUCTURE
   NaN <= '1' when SRC(30 downto 23) = 255 and SRC(22 downto 0) /= 0 else
          '0';
   --Inf <= '1' when SRC(30 downto 0) = 0 else '0';
   Inf <= '1' when SRC(30 downto 23) = 0 else '0';
   Zero <= '1' when SRC(30 downto 0) = "1111111100000000000000000000000" else '0';
   
   S <= SRC(31);				  
   EXP_buf <= 254 - SRC(30 downto 23);
   EXP <= EXP_buf when SRC(22 downto 0) = 0 else EXP_buf - 1;
   
   SMANT0 <= '0' when SRC(22 downto 0) = 0 else '1';
   A <= SMANT0 & SRC(22 downto 0) & '0';
   INV_table7_1 : INV_table7 port map (SRC(22 downto 15),M_buf);
   M_buf1 <= '1' & M_buf & "00000000000000000";
   INV_repeat1 : INV_repeat port map (M_buf1,A,M_buf2);
   INV_repeat2 : INV_repeat port map (M_buf2,A,M_buf3);
   --INV_repeat3 : INV_repeat port map (M_buf3,A,M_buf4);
   
   MANT <= M_buf3(23 downto 1) + 1 when M_buf3(0) = '1' else  --丸め
           M_buf3(23 downto 1);
   
   O <= NaN or (not Zero);
   Os <= (others => O);
   OUTPUT <= --"01111111110000000000000000000000" when NaN = '1' else
             --"00000000000000000000000000000000" when Zero = '1' else
             '0' & Os & "0000000000000000000000" when (NaN or Zero) = '1' else
             S & "1111111100000000000000000000000" when Inf = '1' else
             S & EXP & MANT;
end STRUCTURE;
